from edc_constants.constants import (
    PURPOSIVELY_SELECTED,
    RANDOM_SAMPLING,
    SEQUENTIAL,
    SYSTEMATIC,
)

SELECTION_METHOD = (
    (RANDOM_SAMPLING, "Random sampling"),
    (SYSTEMATIC, "Systematically selected"),
    (SEQUENTIAL, "Sequentially selected"),
    (PURPOSIVELY_SELECTED, "Purposively selected"),
)
